
#include <stdarg.h>

void pr(char *fmt, ... ){
        char buf[128]; // resulting string limited to 128 chars
        va_list args;
        va_start (args, fmt );
        vsnprintf(buf, 128, fmt, args);
        va_end (args);
        Serial.print(buf);

}

// Inputs
// AIN0 (pin 6) - potentiometer
// Analogue 1 - analogue in

// Outputs

// pins which drive the transmit transducer
// PWM on pins 3 and 11 (Timer 2)
#define OUT_PIN0 3
#define OUT_PIN1 11

 
#define AIN0 6
#define AIN1 7



#define BUFFER_SIZE 10


volatile unsigned int buffer[BUFFER_SIZE];
volatile unsigned int tail = 0;

volatile unsigned int icount;

#define ICLIMIT 1



ISR(TIMER1_CAPT_vect) 
{
 if(tail>=BUFFER_SIZE) 
 {
  icount++;
  if(icount>ICLIMIT)
  {
   TIMSK1 &= ~(1<<ICIE1); // stop capture interrupts
   return;
  }
  tail=0;
 }
 buffer[tail++] = ICR1;
}


#define MAXTRANSMIT 21
volatile byte tcount;

ISR(TIMER2_COMPA_vect)
{
 tcount++;
 if(tcount>MAXTRANSMIT)
 {
  TCCR2B = 0; // stop the clock
 } 
}

ISR(TIMER1_OVF_vect)
{
 icount=ICLIMIT+1;
 tcount=MAXTRANSMIT+1;
}


void capture(int enable,int halfperiod)
{
//http://www.openmusiclabs.com/learning/digital/synchronizing-timers/index.html

 GTCCR = (1<<TSM)|(1<<PSRASY)|(1<<PSRSYNC); // halt all timers

// Timer 1 setup

 TCCR1A = 0;    // put timer1 in normal mode 
 TCCR1B = 0; // stop the clock

 TIFR1 = _BV(ICF1) | _BV(TOV1) | _BV(TOV2);  // clear any pending capture or overflow interrupts

 TCNT1H = 0; // set timer1 high byte to 0
 TCNT1L = 0; // set timer1 low byte to 0

 TCCR1B |= (1<< ICES1);   //switch edge detector to detect the rise
 TCCR1B |= (1<< ICNC1);   //noise filter
 TCCR1B |= _BV(CS10);     //free running count

 TIMSK1|=_BV(TOIE1);      // enable overflow interrupt

 TCNT1=0;                 // zero

// Timer 2 setup

 TCNT2 = 0; // set timer2 to 0

 TIMSK2=_BV(OCIE2A); // interrupt on match A

 OCR2A=halfperiod-1;
 OCR2B=halfperiod-1;

 TCCR2A = _BV(COM2A0) | _BV(COM2B0) | _BV(WGM21); // set to toggle OC1A/OC1B on match
 TCCR2B = _BV(CS20); // start the clock, CTC mode, top in OCR1A

   tail=0;
 icount=0;
 tcount=0;

 GTCCR = 0; // release all timers
}



void pwmsetup(void)
{
 pinMode(AIN0, INPUT);
 pinMode(AIN1, INPUT);
 DIDR1  |=  (1<< AIN0D);         // disable digital input buffer AIN0/1
 DIDR1  |=  (1<< AIN1D);

 ADCSRA=0; // ADC Control and Status Register A
 ADCSRB=(1<<ACME); // enable muliplxer
 ADMUX=0; // ADC Multiplexer Selection Register
 ACSR=(1<<ACIC); // Analog Comparator Input Capture by Timer/Counter 1: On

 ADMUX=1;

 pinMode(OUT_PIN0,OUTPUT);
 pinMode(OUT_PIN1,OUTPUT);

 pinMode(5,OUTPUT);

 TCCR2A = 0; // put timer1 in normal mode 
 TCCR2B = 0; // stop the clock
 TCNT2=0;

//toggle OC1A pin as suggested by https://www.avrfreaks.net/comment/748887#comment-748887 
 TCCR2A = _BV(COM2A1) | _BV(COM2B1) | _BV(COM2B0);
 TCCR2B = _BV(FOC2A) | _BV(FOC2B);
}


unsigned long getreading(int halfperiod)
{
 unsigned long sum=0;

 TIMSK0 &= ~_BV(TOIE0); // disable timer0 overflow interrupt
  
 capture(1,halfperiod);
 while(tcount<=MAXTRANSMIT);

 TIMSK1 |= (1<<ICIE1); // enable capture interrupts

 while(icount<=ICLIMIT);

 TIMSK0 |= _BV(TOIE0); // enable timer0 overflow interrupt

 pr("%d",buffer[2]);
 
 return(sum);  
}


void loop() 
{
 getreading(193);
 pr("\n");
 delay(100);
}



void setup() 
{
 Serial.begin(9600);
 while (!Serial) { ; /* wait for serial port to connect. Needed for native USB */}
 pwmsetup();
}


